//+------------------------------------------------------------------+
//|                                                         VWAP.mq5 |
//|                        Copyright 2020, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2020, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"

enum price_METHOD
  {
   Close,
   Open,
   High,
   Low,
   Median,  // Median price (HL/2)
   Typical, // Typical price (HLC/3)
   Weighted // Weighted Close (HLCC/4)
  };
//--- input parameters
input string            vwapID    = "01";       // VWAP ID   ( must be unique )
input price_METHOD      Method    = Typical;    // price Calculation Method
input color             vwapColor = Fuchsia;    // VWAP Color
input int               arrowSize = 3;          // Arrow Size
input ENUM_ARROW_ANCHOR Anchor    = ANCHOR_TOP; // Arrow Anchor Point

input int      InpTakeProfitPts = 100;
input int      InpStopLossPts   = 100;
input double   InpOrderSize     = 0.01;
input string   InpTradeComment  = __FILE__;
input int      InpMagicNumber   = 20000;

double         TakeProfit;
double         StopLoss;
//---buffer identification
const string   IndicatorName = "Anchor VWAP";
const int      IndexVWAP     = 0;
int            Handle;
double         BufferVWAP[];

#include <Trade\Trade.mqh>
CTrade   *Trade;

//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
  Trade = new CTrade();
  Trade.SetExpertMagicNumber(InpMagicNumber);
  
  double point = SymbolInfoDouble(Symbol(), SYMBOL_POINT);
  TakeProfit   = InpTakeProfitPts * point;
  StopLoss     = InpStopLossPts * point;

  Handle = iCustom(Symbol(), 0, IndicatorName, vwapID, Method, 
                  vwapColor, arrowSize, Anchor);
                
  if (Handle==INVALID_HANDLE) 
  {
      PrintFormat("Error %i", GetLastError());
      return(INIT_FAILED);
  }
  if(!AddIndicator())
     {
      //--- AddIndicator() function refused to add the indicator on the chart
      int answer=MessageBox("Do you want to add VWAP on the chart anyway?",
                            "Incorrect symbol and/or time frame for adding the indicator",
                            MB_YESNO // "Yes" and "No" selection buttons will be shown
                            );
      //--- if a user still insists on incorrect usage of ChartIndicatorAdd()
      if(answer==IDYES)
        {
         //--- first of all, a Journal entry will be made about that
         PrintFormat("Attention! %s: Trying to add VWAP(%s/%s) indicator on %s/%s chart. Receiving error 4114",
                     __FUNCTION__,Symbol(),EnumToString(Period()),_Symbol,EnumToString(_Period));
         //--- receive the number of a new subwindow, to which we will try to add the indicator
         int subwindow=(int)ChartGetInteger(0,CHART_WINDOWS_TOTAL);
         //--- now make an attempt resulting in error
         if(!ChartIndicatorAdd(0,subwindow,Handle))
            PrintFormat("Failed to add MACD indicator on %d chart window. Error code  %d",
                        subwindow,GetLastError());
        }
       }
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
  IndicatorRelease(Handle);
  delete Trade; 
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
   ArraySetAsSeries(BufferVWAP, true);
   if (CopyBuffer(Handle, IndexVWAP, 0, 1, BufferVWAP) < 0) {Print("CopyBufferMA1 error =",GetLastError());}
   
   MqlTick Latest_price; // Structure to get the latest prices      
   SymbolInfoTick(Symbol() ,Latest_price); // Assign current prices to structure 

// The BID price.
   static double dBid_price; 

// The ASK price.
   static double dAsk_price; 

   dBid_price = Latest_price.bid;  // Current Bid price.
   dAsk_price = Latest_price.ask;  // Current Ask price.
   
   double  currentVWAP = BufferVWAP[0];
   
// Execute strategy
   bool  buyCondition  = (dBid_price == currentVWAP);
   bool  sellCondition = (dAsk_price == currentVWAP);
   
   if (buyCondition) {
      OrderOpen(ORDER_TYPE_BUY);
   } else
   if (sellCondition) {
      OrderOpen(ORDER_TYPE_SELL);
   }
   return;
   

  }
  
bool OrderOpen(ENUM_ORDER_TYPE orderType)
{
  double price;
  double stopLossPrice;
  double takeProfitPrice;
  
  if (orderType==ORDER_TYPE_BUY)
  {
      price            =   NormalizeDouble(SymbolInfoDouble(Symbol(), SYMBOL_ASK), Digits());
      stopLossPrice    =   NormalizeDouble(price - StopLoss, Digits()) ;
      takeProfitPrice  =   NormalizeDouble(price + TakeProfit, Digits());
  } else if (orderType==ORDER_TYPE_SELL)
  {
      price            =   NormalizeDouble(SymbolInfoDouble(Symbol(), SYMBOL_BID), Digits());
      stopLossPrice    =   NormalizeDouble(price + StopLoss, Digits()) ;
      takeProfitPrice  =   NormalizeDouble(price - TakeProfit, Digits());
  } else return(false);
  
  Trade.PositionOpen(Symbol(), orderType, InpOrderSize, price, stopLossPrice, takeProfitPrice, InpTradeComment);

  return(true);
 
}
//+------------------------------------------------------------------+
bool AddIndicator()
  {
//--- displayed message
   string message;
//--- check if the indicator symbol and chart symbol match each other
   if(Symbol()!=_Symbol)
     {
      message="Displaying the use of Demo_ChartIndicatorAdd() function:";
      message=message+"\r\n";
      message=message+"Unable to add the indicator calculated on another symbol on the chart.";
      message=message+"\r\n";
      message=message+"Specify the chart symbol in Expert Advisor's property - "+_Symbol+".";
      Alert(message);
      //--- premature exit, the indicator will not be added on the chart
      return false;
     }
//--- check if the indicator's and chart's time frames match each other
   if(Period()!=_Period)
     {
      message="Unable to add the indicator calculated on another time frame on the chart.";
      message=message+"\r\n";
      message=message+"Specify the chart time frame in Expert Advisor properties - "+EnumToString(_Period)+".";
      Alert(message);
      //--- premature exit, the indicator will not be added on the chart
      return false;
     }
//--- apply the indicator to the chart 
   Print(__FUNCTION__,"  Adding VWAP indicator on the chart");
   Print("VWAP is generated on ",Symbol(),"/",EnumToString(Period()));
//--- receive the number of a new subwindow, to which MACD indicator is added
   int subwindow=(int)ChartGetInteger(0,CHART_WINDOWS_TOTAL);
   PrintFormat("Adding VWAP indicator on %d chart window",subwindow);
   if(!ChartIndicatorAdd(0,subwindow,Handle))
     {
      PrintFormat("Failed to add MACD indicator on %d chart window. Error code  %d",
                  subwindow,GetLastError());
     }
//--- Indicator added successfully
   return(true);
   }